<?php
/**
 * This is used to save settings.
 *
 * @package    Mindd_Widget
 * @subpackage Mindd_Widget/admin
 */
class Mindd_Widget_Settings {

	/**
	 * Holds the values to be used in the fields callbacks.
	 *
	 * @var array<mixed>
	 */
	private $options;

	/**
	 * Holds the sections to be used in the settings page.
	 *
	 * @var array<mixed>
	 */
	private $sections;

	/**
	 * Holds the languages to be used in the settings page.
	 *
	 * @var array<mixed>
	 */
	private $languages;

	/**
	 * Start up
	 */
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
		add_action( 'admin_init', array( $this, 'page_init' ) );

		$this->languages = array(
			'NL' => __( 'Dutch', 'mindd-widget' ),
			'BE' => __( 'Belgian', 'mindd-widget' ),
			'DE' => __( 'German', 'mindd-widget' ),
			'EN' => __( 'English', 'mindd-widget' ),
			'FR' => __( 'French', 'mindd-widget' ),
			'PL' => __( 'Polish', 'mindd-widget' ),
			'TR' => __( 'Turkish', 'mindd-widget' ),
			'RU' => __( 'Russian', 'mindd-widget' ),
			'UK' => __( 'Ukrainian', 'mindd-widget' ),
			'AR' => __( 'Arabic', 'mindd-widget' ),
		);

		$post_id = isset( $_REQUEST['post'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['post'] ) ) : false; // phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$post_id = intval( $post_id );

		if ( get_post_type( $post_id ) === 'mindd-widget' ) {
			if ( get_post_meta( $post_id, 'mindd_widget', true ) ) {
				$this->options = get_post_meta( $post_id, 'mindd_widget', true );
			} else {
				$this->options = get_option( 'mindd_widget' );
			}
		} else {
			$this->options = get_option( 'mindd_widget' );
		}
	}

	/**
	 * Add options page.
	 *
	 * @return void
	 */
	public function add_plugin_page() {
		// This page will be under "Settings".
		add_options_page(
			'MINDD Widget',
			'MINDD Widget',
			'manage_options',
			'mindd-widget',
			array( $this, 'create_admin_page' )
		);
	}

	/**
	 * Options page callback.
	 *
	 * @return void
	 */
	public function create_admin_page() {
		?>
		<div class="wrap">
			<h1><?php esc_html_e( 'MINDD Widget', 'mindd-widget' ); ?></h1>

			<form method="post" action="options.php">
				<?php settings_fields( 'mindd_widget_settings_group' ); ?>

				<div id="api-settings">
					<?php do_settings_sections( 'mindd_widget_api_settings_section' ); ?>
				</div>
				<div id="display-settings">
					<?php do_settings_sections( 'mindd_widget_display_settings_section' ); ?>
				</div>
				<div id="language-settings">
					<?php do_settings_sections( 'mindd_widget_language_settings_section' ); ?>
				</div>

				<?php submit_button(); ?>
			</form>
		</div>
		<?php
	}

	/**
	 * Register and add settings.
	 *
	 * @return void
	 */
	public function page_init() {
		register_setting(
			'mindd_widget_settings_group', // Option group.
			'mindd_widget', // Option name.
			array( $this, 'sanitize' ) // Sanitize.
		);

		$this->sections = array(
			array(
				'id'     => 'mindd_widget_api_settings_section',
				'title'  => __( 'API', 'mindd-widget' ),
				'fields' => array(
					array(
						'title'    => __( 'MINDD API Key', 'mindd-widget' ) . ' <span class="acf-required">*</span>',
						'callback' => array( $this, 'required_text_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_api_key',
							'description' => __( 'Your own key as provided to you by MINDD.', 'mindd-widget' ),
							'placeholder' => '',
						),
					),
				),
			),
			array(
				'id'     => 'mindd_widget_display_settings_section',
				'title'  => __( 'Display', 'mindd-widget' ),
				'fields' => array(
					array(
						'title'    => __( 'Header', 'mindd-widget' ),
						'callback' => array( $this, 'radio_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_layout',
							'description' => __( 'Would you like to display a title bar above the widget showing your practice name or logo?', 'mindd-widget' ),
							'placeholder' => '',
							'options'     => array(
								'top' => __( 'Yes', 'mindd-widget' ),
								null  => __( 'No', 'mindd-widget' ),
							),
						),
					),

					array(
						'title'    => __( 'Display mode', 'mindd-widget' ),
						'callback' => array( $this, 'radio_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_modal_display_mode',
							'description' => __( 'Do you want to show the opened widget full screen or as a dialog?', 'mindd-widget' ),
							'placeholder' => '',
							'options'     => array(
								'full'   => __( 'Fullscreen', 'mindd-widget' ),
								'dialog' => __( 'Dialog', 'mindd-widget' ),
							),
						),
					),
					array(
						'title'    => __( 'Accent Color', 'mindd-widget' ),
						'callback' => array( $this, 'color_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_accent_color',
							'description' => __( 'What color are the buttons?', 'mindd-widget' ),
							'placeholder' => '',
						),
					),
					array(
						'title'    => __( 'Background Color', 'mindd-widget' ),
						'callback' => array( $this, 'color_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_background',
							'description' => __( 'What will the background color be?', 'mindd-widget' ),
							'placeholder' => '',
						),
					),
					array(
						'title'    => __( 'Foreground Color', 'mindd-widget' ),
						'callback' => array( $this, 'color_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_foreground',
							'description' => __( 'Wat wordt de kleur van de welkomsttekst?', 'mindd-widget' ),
							'placeholder' => '',
						),
					),
					array(
						'title'    => __( 'Font Family', 'mindd-widget' ),
						'callback' => array( $this, 'select_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_font_family',
							'options'     => array(
								'Montserrat' => __( 'Montserrat', 'mindd-widget' ),
								'Arial'      => __( 'Arial', 'mindd-widget' ),
								'Verdana'    => __( 'Verdana', 'mindd-widget' ),
								'Calibri'    => __( 'Calibri', 'mindd-widget' ),
								'Roboto'     => __( 'Roboto', 'mindd-widget' ),
							),
							'description' => __( 'Choose the font used in the widget.', 'mindd-widget' ),
							'placeholder' => '',
						),
					),
				),
			),
			array(
				'id'     => 'mindd_widget_language_settings_section',
				'title'  => __( 'Language', 'mindd-widget' ),
				'fields' => array(
					array(
						'title'    => __( 'Language Selector', 'mindd-widget' ),
						'callback' => array( $this, 'radio_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_show_language_selector',
							'description' => __( 'Do you want to offer the widget in multiple languages?', 'mindd-widget' ),
							'placeholder' => '',
							'options'     => array(
								'true'  => __( 'Yes', 'mindd-widget' ),
								'false' => __( 'No', 'mindd-widget' ),
							),
						),
					),
					array(
						'title'    => __( 'Default Language', 'mindd-widget' ),
						'callback' => array( $this, 'select_callback' ),
						'args'     => array(
							'id'          => 'mindd_widget_default_language_code',
							'description' => __( 'What language is the default in the widget?', 'mindd-widget' ),
							'placeholder' => '',
							'options'     => $this->languages,
						),
					),
				),
			),
		);

		foreach ( $this->sections as $section ) {
			$default_args = array(
				'callback' => array( $this, 'section_callback' ),
				'page'     => strval( $section['id'] ),
				'fields'   => array(),
				'args'     => array(),
			);

			$section = array_merge( $default_args, $section );

			add_settings_section( strval( $section['id'] ), strval( $section['title'] ), $section['callback'], strval( $section['page'] ), $section['args'] );

			if ( is_array( $section['fields'] ) ) :
				foreach ( $section['fields'] as $field ) {
					$default_args = array(
						'callback' => '__return_false',
						'page'     => strval( $section['id'] ),
						'section'  => $section['id'],
						'args'     => array(),
					);

					if ( is_array( $field ) ) :
						$field = array_merge( $default_args, $field );

						add_settings_field( $field['args']['id'], $field['title'], $field['callback'], $field['page'], $field['section'], $field['args'] );
					endif;
				}
			endif;
		}
	}

	/**
	 * Sanitize each setting field as needed.
	 *
	 * @param array<mixed> $input Contains all settings fields as array keys.
	 *
	 * @return array<mixed>
	 */
	public function sanitize( $input ) {
		$new_input = $input;

		return $new_input;
	}

	/**
	 * Renders a set of radio buttons.
	 *
	 * @param array<mixed> $args {
	 *     An array of arguments.
	 *
	 *     @type array    $options     The options for the radio buttons.
	 *     @type string   $id          The ID of the radio button group.
	 *     @type string   $description The description of the radio button group.
	 * }
	 *
	 * @return void
	 */
	public function radio_callback( $args ) {
		$options     = $args['options'];
		$id          = $args['id'];
		$description = $args['description'];

		if ( isset( $this->options[ $id ] ) ) {
			$selected_option = $this->options[ $id ];
		} else {
			$selected_option = '';
		}

		$html_options = '';

		foreach ( $options as $key => $value ) {
			$html_options .= sprintf(
				'<p><label><input type="radio" name="mindd_widget[%1$s]" value="%2$s" %3$s> %4$s</label></p>',
				esc_attr( $id ),
				esc_attr( $key ),
				checked( $selected_option, $key, false ),
				__( $value, 'mindd-widget' ) // phpcs:ignore WordPress.WP.I18n.NonSingularStringLiteralText
			);
		}

		echo '<p>' . esc_html( $description ) . '</p>';

		echo $html_options; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

		
	}

	/**
	 * Renders a select field.
	 *
	 * @param array<mixed> $args {
	 *     An array of arguments.
	 *
	 *     @type array    $options     The options for the select field.
	 *     @type string   $id          The ID of the select field.
	 *     @type string   $description The description of the select field.
	 * }
	 *
	 * @return void
	 */
	public function select_callback( $args ) {
		$options     = $args['options'];
		$id          = $args['id'];
		$description = $args['description'];

		if ( isset( $this->options[ $id ] ) ) {
			$selected_option = $this->options[ $id ];
		} else {
			$selected_option = '';
		}

		$html_options = '';

		foreach ( $options as $key => $value ) {
			$html_options .= sprintf(
				'<option value="%1$s" %3$s>%2$s</option>',
				esc_attr( $key ),
				__( $value, 'mindd-widget' ), // phpcs:ignore WordPress.WP.I18n.NonSingularStringLiteralText
				selected( $selected_option, esc_attr( $key ), false )
			);
		}

		echo '<p>' . esc_html( $description ) . '</p>';

		echo '<select name="mindd_widget[' . esc_attr( $id ) . ']" class="regular-text">' . $html_options . '</select>'; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

		
	}

	/**
	 * Renders a text field.
	 *
	 * @param array<mixed> $args {
	 *     An array of arguments.
	 *
	 *     @type string   $id          The ID of the text field.
	 *     @type string   $description The description of the text field.
	 * }
	 *
	 * @return void
	 */
	public function text_callback( $args ) {
		$id          = $args['id'];
		$description = $args['description'];
		$placeholder = $args['placeholder'];

		echo '<p>' . esc_html( $description ) . '</p>';

		printf(
			'<input type="text" id="' . $id . '" name="mindd_widget[' . $id . ']" value="%s" placeholder="' . $placeholder . '" class="regular-text" />',
			isset( $this->options[ $id ] ) ? esc_attr( $this->options[ $id ] ) : ''
		);

		
	}

	/**
	 * Renders a colorpicker field.
	 *
	 * @param array<mixed> $args {
	 *     An array of arguments.
	 *
	 *     @type string   $id          The ID of the text field.
	 *     @type string   $description The description of the text field.
	 * }
	 *
	 * @return void
	 */
	public function color_callback( $args ) {
		$id          = $args['id'];
		$description = $args['description'];
		$placeholder = $args['placeholder'];

		echo '<p>' . esc_html( $description ) . '</p>';

		printf(
			'<input type="text" id="' . $id . '" name="mindd_widget[' . $id . ']" value="%s" placeholder="' . $placeholder . '" class="regular-text wp-colorpicker" />',
			isset( $this->options[ $id ] ) ? esc_attr( $this->options[ $id ] ) : ''
		);

		
	}

	/**
	 * Renders a required text field.
	 *
	 * @param array<mixed> $args {
	 *     An array of arguments.
	 *
	 *     @type string   $id          The ID of the text field.
	 *     @type string   $description The description of the text field.
	 * }
	 *
	 * @return void
	 */
	public function required_text_callback( $args ) {
		$id          = $args['id'];
		$description = $args['description'];
		$placeholder = $args['placeholder'];

		echo '<p>' . esc_html( $description ) . '</p>';

		printf(
			'<input type="text" id="' . $id . '" required name="mindd_widget[' . $id . ']" value="%s" placeholder="' . $placeholder . '" class="regular-text" />',
			isset( $this->options[ $id ] ) ? esc_attr( $this->options[ $id ] ) : ''
		);

		
	}

	/**
	 * Renders the description text of a section.
	 *
	 * @param array<mixed> $args {
	 *     An array of arguments.
	 *
	 *     @type string   $description The description of the section.
	 * }
	 *
	 * @return void
	 */
	public function section_callback( $args ) {
	}
}

if ( is_admin() ) {
	new Mindd_Widget_Settings();
}
