<?php

/**
 * Register all actions and filters for the plugin.
 *
 * Maintain a list of all hooks that are registered throughout the plugin, and register them with the WordPress API.
 * Call the run function to execute the list of actions and filters.
 *
 * @package    Mindd_Widget
 * @subpackage Mindd_Widget/includes
 */
class Mindd_Widget_Loader {

	/**
	 * The array of actions registered with WordPress.
	 *
	 * @var array<mixed>
	 */
	protected $actions;

	/**
	 * The array of filters registered with WordPress.
	 *
	 * @var array<mixed>
	 */
	protected $filters;

	/**
	 * Initialize the collections used to maintain the actions and filters.
	 */
	public function __construct() {
		$this->actions = array();
		$this->filters = array();
	}

	/**
	 * Add a new action to the collection to be registered with WordPress.
	 *
	 * @param string $hook The name of the action to add the callback to.
	 * @param object $component The class to be used when the action is called.
	 * @param string $callback The callback to be run when the action is called.
	 * @param int    $priority Used to specify the order in which the functions associated with a particular action are executed.
	 * @param int    $accepted_args The number of arguments the function accepts.
	 *
	 * @return void
	 */
	public function add_action( $hook, $component, $callback, $priority = 10, $accepted_args = 1 ) {
		$this->actions = $this->add( $this->actions, $hook, $component, $callback, $priority, $accepted_args );
	}

	/**
	 * Add a new filter to the collection to be registered with WordPress.
	 *
	 * @param string $hook The name of the filter to add the callback to.
	 * @param object $component The class to be used when the filter is called.
	 * @param string $callback The callback to be run when the filter is called.
	 * @param int    $priority Used to specify the order in which the functions associated with a particular filter are executed.
	 * @param int    $accepted_args The number of arguments the function accepts.
	 *
	 * @return void
	 */
	public function add_filter( $hook, $component, $callback, $priority = 10, $accepted_args = 1 ) {
		$this->filters = $this->add( $this->filters, $hook, $component, $callback, $priority, $accepted_args );
	}

	/**
	 * A utility function that is used to register the actions and hooks into a single collection.
	 *
	 * @param array<mixed> $hooks The array of actions and filters registered with WordPress.
	 * @param string       $hook The name of the action or filter to add the callback to.
	 * @param object       $component The class to be used when the action or filter is called.
	 * @param string       $callback The callback to be run when the action or filter is called.
	 * @param int          $priority Used to specify the order in which the functions associated with a particular action or filter are executed.
	 * @param int          $accepted_args The number of arguments the function accepts.
	 *
	 * @return array<mixed>
	 */
	private function add( $hooks, $hook, $component, $callback, $priority, $accepted_args ) {
		$hooks[] = array(
			'hook'          => $hook,
			'component'     => $component,
			'callback'      => $callback,
			'priority'      => $priority,
			'accepted_args' => $accepted_args,
		);

		return $hooks;
	}

	/**
	 * Register the filters and actions with WordPress.
	 *
	 * @return void
	 */
	public function run() {
		foreach ( $this->filters as $hook ) {
			add_filter( $hook['hook'], array( $hook['component'], $hook['callback'] ), $hook['priority'], $hook['accepted_args'] );
		}

		foreach ( $this->actions as $hook ) {
			add_action( $hook['hook'], array( $hook['component'], $hook['callback'] ), $hook['priority'], $hook['accepted_args'] );
		}
	}

}
