<?php

/**
 * The class allows to define and register custom post types.
 *
 * This class provides an easy and flexible way to define custom post types with various options and features, such as custom labels, taxonomies, meta boxes, and more.
 * Developers can use this class to create custom post types that fit their specific needs and requirements, and to extend the functionality of WordPress beyond the default post types.
 *
 * @package    Mindd_Widget
 * @subpackage Mindd_Widget/includes
 */
class Mindd_Widget_Post_Type {

	/**
	 * The post type slug.
	 *
	 * @var string
	 */
	private $post_type;

	/**
	 * The post type arguments.
	 *
	 * @var array<mixed>
	 */
	private $arguments;

	/**
	 * The post type labels.
	 *
	 * @var array<mixed>
	 */
	private $labels;

	/**
	 * The singular label for the post type.
	 *
	 * @var string
	 */
	private $singular_label;

	/**
	 * The plural label for the post type.
	 *
	 * @var string
	 */
	private $plural_label;

	/**
	 * Holds the languages to be used in the settings page.
	 *
	 * @var array<mixed>
	 */
	private $languages;

	/**
	 * Constructor for the post type class.
	 *
	 * @param string $post_type The post type slug.
	 */
	public function __construct( $post_type ) {
		$this->set_post_type( $post_type );
		$this->set_singular_label( ucfirst( $this->get_post_type() ) );
		$this->set_plural_label( ucfirst( $this->get_post_type() ) . 's' );

		$this->languages = array(
			'NL' => __( 'Dutch', 'mindd-widget' ),
			'BE' => __( 'Belgian', 'mindd-widget' ),
			'DE' => __( 'German', 'mindd-widget' ),
			'EN' => __( 'English', 'mindd-widget' ),
			'FR' => __( 'French', 'mindd-widget' ),
			'PL' => __( 'Polish', 'mindd-widget' ),
			'TR' => __( 'Turkish', 'mindd-widget' ),
			'RU' => __( 'Russian', 'mindd-widget' ),
			'UK' => __( 'Ukrainian', 'mindd-widget' ),
			'AR' => __( 'Arabic', 'mindd-widget' ),
		);
	}

	/**
	 * Sets the post type slug.
	 *
	 * @param string $post_type The post type slug.
	 *
	 * @return void
	 */
	public function set_post_type( $post_type ) {
		$this->post_type = $post_type;
	}

	/**
	 * Gets the post type slug.
	 *
	 * @return string The post type slug.
	 */
	public function get_post_type() {
		return $this->post_type;
	}

	/**
	 * Sets the post type arguments.
	 *
	 * @param array<mixed> $arguments The post type arguments.
	 *
	 * @return void
	 */
	public function set_arguments( $arguments = array() ) {
		$default_arguments = array(
			'public'   => false,
			'show_ui'  => true,
			'labels'   => $this->get_labels(),
			'supports' => array(
				'title',
				'revisions',
			),
		);

		$this->arguments = array_merge( $default_arguments, $arguments );
	}

	/**
	 * Gets the post type arguments.
	 *
	 * @return array<mixed> The post type arguments.
	 */
	public function get_arguments() {
		return $this->arguments;
	}

	/**
	 * Sets the singular label of the post type.
	 *
	 * @param string $singular_label The singular label of the post type.
	 *
	 * @return void
	 */
	public function set_singular_label( $singular_label ) {
		$this->singular_label = $singular_label;
	}

	/**
	 * Gets the singular label of the post type.
	 *
	 * @return string The singular label of the post type.
	 */
	public function get_singular_label() {
		return $this->singular_label;
	}

	/**
	 * Sets the plural label of the post type.
	 *
	 * @param string $plural_label The plural label of the post type.
	 *
	 * @return void
	 */
	public function set_plural_label( $plural_label ) {
		$this->plural_label = $plural_label;
	}

	/**
	 * Gets the plural label of the post type.
	 *
	 * @return string The plural label of the post type.
	 */
	public function get_plural_label() {
		return $this->plural_label;
	}

	/**
	 * Sets the labels for the post type.
	 *
	 * @param array<mixed> $labels The labels for the post type.
	 *
	 * @return void
	 */
	public function set_labels( $labels ) {
		$default_labels = array(
			'name'                     => $this->get_plural_label(),
			'singular_name'            => $this->get_singular_label(),
			'add_new'                  => 'Add New',
			'add_new_item'             => 'Add New ' . $this->get_singular_label(),
			'edit_item'                => 'Edit ' . $this->get_singular_label(),
			'new_item'                 => 'New ' . $this->get_singular_label(),
			'view_item'                => 'View ' . $this->get_singular_label(),
			'view_items'               => 'View ' . $this->get_plural_label(),
			'search_items'             => 'Search ' . $this->get_plural_label(),
			'not_found'                => 'No ' . lcfirst( $this->get_plural_label() ) . ' found',
			'not_found_in_trash'       => 'No ' . lcfirst( $this->get_plural_label() ) . ' found in Trash',
			'parent_item_colon'        => 'Parent ' . $this->get_singular_label() . ':',
			'all_items'                => 'All ' . $this->get_plural_label(),
			'archives'                 => $this->get_singular_label() . ' Archives',
			'attributes'               => $this->get_singular_label() . ' Attributes',
			'insert_into_item'         => 'Insert into ' . lcfirst( $this->get_singular_label() ),
			'uploaded_to_this_item'    => 'Uploaded to this ' . lcfirst( $this->get_singular_label() ),
			'featured_image'           => 'Featured image',
			'set_featured_image'       => 'Set featured image',
			'remove_featured_image'    => 'Remove featured image',
			'use_featured_image'       => 'Use as featured image',
			'menu_name'                => $this->get_plural_label(),
			'filter_items_list'        => 'Filter ' . lcfirst( $this->get_plural_label() ) . ' list',
			'filter_by_date'           => 'Filter by date',
			'items_list_navigation'    => $this->get_plural_label() . ' list navigation',
			'items_list'               => $this->get_plural_label() . ' list',
			'item_published'           => $this->get_singular_label() . ' published.',
			'item_published_privately' => $this->get_singular_label() . ' published privately.',
			'item_reverted_to_draft'   => $this->get_singular_label() . ' reverted to draft.',
			'item_scheduled'           => $this->get_singular_label() . ' scheduled.',
			'item_updated'             => $this->get_singular_label() . ' updated.',
			'item_link'                => $this->get_singular_label() . ' Link',
			'item_link_description'    => 'A link to a ' . lcfirst( $this->get_singular_label() ) . '.',
		);

		$this->labels = array_merge( $default_labels, $labels );
	}

	/**
	 * Gets the labels for the post type.
	 *
	 * @return array<mixed> $labels The labels for the post type.
	 */
	public function get_labels() {
		return $this->labels;
	}

	/**
	 * Registers the custom post type.
	 *
	 * @param array<mixed> $arguments Optional. Arguments to customize the post type registration.
	 * @param array<mixed> $labels Optional. Labels for the post type.
	 * @return void
	 */
	public function register( $arguments = array(), $labels = array() ) {
		$this->set_labels( $labels );
		$this->set_arguments( $arguments );

		register_post_type( $this->get_post_type(), $this->get_arguments() );
	}

	/**
	 * Adds a new admin column to the post type list table.
	 *
	 * @param string $column The column name.
	 * @param string $title The column title.
	 * @return void
	 */
	public function add_admin_column( $column, $title ) {
		add_filter(
			'manage_' . $this->get_post_type() . '_posts_columns',
			function( $columns ) use ( $column, $title ) {
				$columns[ $column ] = $title;

				return $columns;
			}
		);
	}

	/**
	 * Removes an admin column from the post type list table.
	 *
	 * @param string $column The name of the column to remove.
	 * @return void
	 */
	public function remove_admin_column( $column ) {
		add_filter(
			'manage_' . $this->get_post_type() . '_posts_columns',
			function( $columns ) use ( $column ) {
				unset( $columns[ $column ] );

				return $columns;
			}
		);
	}

	/**
	 * Sets the title for an admin column in the post type list table.
	 *
	 * @param string $column The name of the column to set the title for.
	 * @param string $title The title to set for the column.
	 * @return void
	 */
	public function set_admin_column_title( $column, $title ) {
		add_filter(
			'manage_' . $this->get_post_type() . '_posts_columns',
			function( $columns ) use ( $column, $title ) {
				$columns[ $column ] = $title;

				return $columns;
			}
		);
	}

	/**
	 * Sets the content for an admin column in the post type list table.
	 *
	 * @param string       $column    The name of the admin column to set the content for.
	 * @param mixed        $function  The function or method to call to generate the content for the column.
	 * @param array<mixed> $arguments An optional array of arguments to pass to the function or method.
	 *                         If provided, the arguments will be passed as an array using `call_user_func_array`.
	 *                         If not provided, the function or method will be called with no arguments using `call_user_func`.
	 * @return void
	 */
	public function set_admin_column_content( $column, $function, $arguments = array() ) {
		add_action(
			'manage_' . $this->get_post_type() . '_posts_custom_column',
			function ( $column_name, $post_id ) use ( $column, $function, $arguments ) {
				if ( $column_name === $column ) {
					if ( is_array( $arguments ) && count( $arguments ) > 0 ) {
						call_user_func_array( $function, $arguments );
					} else {
						call_user_func( $function );
					}
				}
			},
			10,
			2
		);
	}

	/**
	 * Makes an admin column sortable in the post type list table.
	 *
	 * @param string $column The name of the column to make sortable.
	 * @return void
	 */
	public function make_admin_column_sortable( $column ) {
		add_filter(
			'manage_edit-' . $this->get_post_type() . '_sortable_columns',
			function( $columns ) use ( $column ) {
				$columns[ $column ] = $column;

				return $columns;
			}
		);
	}

	/**
	 * Modifies the query for a sortable admin column in the post type list table.
	 *
	 * @param string $column The name of the column to sort by.
	 * @param string $query_var The name of the query variable to modify.
	 * @param string $value The value to set the query variable to.
	 *
	 * @return void
	 */
	public function modify_admin_column_query( $column, $query_var, $value ) {
		add_action(
			'pre_get_posts',
			function ( $query ) use ( $column, $query_var, $value ) {
				if ( ! is_admin() || ! $query->is_main_query() || $query->get( 'post_type' ) !== $this->get_post_type() ) {
					return;
				}

				$order_by = $query->query['orderby'];

				if ( $order_by === $column ) {
					$query->set( $query_var, $value );
				}
			}
		);
	}

	/**
	 * Reorders the admin columns in the post type list table.
	 *
	 * @param array<string> $order Array of column names in the order they should appear.
	 *
	 * @return void
	 */
	public function reorder_admin_columns( $order ) {
		add_filter(
			'manage_' . $this->get_post_type() . '_posts_columns',
			function ( $columns ) use ( $order ) {
				$ordered_columns = array();

				foreach ( $order as $column_name ) {
					if ( isset( $columns[ $column_name ] ) ) {
						$ordered_columns[ $column_name ] = $columns[ $column_name ];
						unset( $columns[ $column_name ] );
					}
				}

				$columns = array_merge( $ordered_columns, $columns );

				return $columns;
			}
		);
	}

	/**
	 * Sets the template folder for the custom post type.
	 *
	 * @param string $folder The folder containing the templates.
	 *
	 * @return void
	 */
	public function set_template_folder( $folder ) {
		add_action(
			'single_template',
			function ( $template ) use ( $folder ) {
				global $post;

				if ( 'hap_v1' === get_option( 'stylesheet' ) ) {
					return $template;
				}

				if ( $this->get_post_type() === $post->post_type && locate_template( array( 'single-' . $this->get_post_type() . '.php' ) ) !== $template ) {
					if ( file_exists( plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/single-' . $this->get_post_type() . '.php' ) ) {
						return plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/single-' . $this->get_post_type() . '.php';
					} else {
						return $template;
					}
				}

				if ( $this->get_post_type() === $post->post_type && ! $template ) {
					if ( file_exists( plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/single-' . $this->get_post_type() . '.php' ) ) {
						return plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/single-' . $this->get_post_type() . '.php';
					} else {
						return $template;
					}
				}

				return $template;
			}
		);

		add_action(
			'archive_template',
			function ( $template ) use ( $folder ) {
				global $post;

				if ( 'hap_v1' === get_option( 'stylesheet' ) ) {
					return $template;
				}

				if ( ! isset( $post->post_type ) ) {
					return $template;
				}

				if ( $this->get_post_type() === $post->post_type && locate_template( array( 'archive-' . $this->get_post_type() . '.php' ) ) !== $template ) {
					if ( file_exists( plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/archive-' . $this->get_post_type() . '.php' ) ) {
						return plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/archive-' . $this->get_post_type() . '.php';
					} else {
						return $template;
					}
				}

				if ( $this->get_post_type() === $post->post_type && ! $template ) {
					if ( file_exists( plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/archive-' . $this->get_post_type() . '.php' ) ) {
						return plugin_dir_path( dirname( __FILE__ ) ) . $folder . '/archive-' . $this->get_post_type() . '.php';
					} else {
						return $template;
					}
				}

				return $template;
			}
		);
	}

	/**
	 * Modifies the query for the post type list table.
	 *
	 * @param string $query_var The name of the query variable to modify.
	 * @param string $value The value to set the query variable to.
	 *
	 * @return void
	 */
	public function modify_admin_query( $query_var, $value ) {
		add_action(
			'pre_get_posts',
			function ( $query ) use ( $query_var, $value ) {
				if ( ! is_admin() || ! $query->is_main_query() || $query->get( 'post_type' ) !== $this->get_post_type() || ! empty( $query->query['orderby'] ) ) {
					return;
				}

				$query->set( $query_var, $value );
			}
		);
	}

	/**
	 * Modifies the query for the post type archive.
	 *
	 * @param string $query_var The name of the query variable to modify.
	 * @param string $value The value to set the query variable to.
	 *
	 * @return void
	 */
	public function modify_query( $query_var, $value ) {
		add_action(
			'pre_get_posts',
			function ( $query ) use ( $query_var, $value ) {
				if ( is_admin() || ! $query->is_main_query() || $query->get( 'post_type' ) !== $this->get_post_type() ) {
					return;
				}

				$query->set( $query_var, $value );
			}
		);
	}

	/**
	 * Adds meta boxes for the MINDD widget settings.
	 *
	 * @return void
	 */
	public function add_meta_box() {
		add_meta_box( 'mindd_widget_settings', esc_html__( 'MINDD Widget Settings', 'mindd-widget' ), array( $this, 'render_meta_box' ), $this->post_type, 'advanced', 'high' );

		$post_types = get_post_types(
			array(
				'public' => true,
			)
		);

		add_meta_box( 'mindd_widget_settings', esc_html__( 'MINDD Widget', 'mindd-widget' ), array( $this, 'render_post_meta_box' ), $post_types, 'advanced', 'high' );
	}

	/**
	 * Render the post meta box for MINDD widgets.
	 *
	 * This function displays a dropdown list of available MINDD widgets for selection.
	 *
	 * @return void
	 */
	public function render_post_meta_box() {
		$args = array(
			'showposts' => -1,
			'post_type' => 'mindd-widget',
		);

		$widgets         = get_posts( $args );
		$post_id         = intval( get_the_ID() );
		$mindd_widget_id = get_post_meta( $post_id, 'mindd_widget_id', true );

		wp_nonce_field( 'mindd_widget_meta_box', 'mindd_widget_nonce' );
		?>
		<div class="wrap">
			<select name="mindd_widget_id">
				<option>Default</option>
				<?php foreach ( $widgets as $widget ) : ?>
					<option value="<?php echo esc_attr( strval( $widget->ID ) ); ?>" <?php selected( $widget->ID, $mindd_widget_id ); ?>><?php echo esc_html( $widget->post_title ); ?></option>
				<?php endforeach; ?>
			</select>
		</div>
		<?php
	}

	/**
	 * Render the meta box for MINDD widget settings.
	 *
	 * This function displays a set of tabs with different settings sections for API, Display, Texts, and Branding.
	 * Each tab corresponds to a specific settings section that gets displayed accordingly.
	 *
	 * @return void
	 */
	public function render_meta_box() {
		wp_nonce_field( 'mindd_widget_meta_box', 'mindd_widget_nonce' );
		?>
		<div class="wrap">
			<nav class="nav-tab-wrapper">
				<a class="nav-tab nav-tab-active" href="#api-settings"><?php esc_html_e( 'API', 'mindd-widget' ); ?></a>
				<a class="nav-tab" href="#display-settings"><?php esc_html_e( 'Display', 'mindd-widget' ); ?></a>
				<a class="nav-tab" href="#text-settings"><?php esc_html_e( 'Texts', 'mindd-widget' ); ?></a>
				<a class="nav-tab" href="#branding-settings"><?php esc_html_e( 'Branding', 'mindd-widget' ); ?></a>
			</nav>
			<div id="api-settings">
				<?php do_settings_sections( 'mindd_widget_api_settings_section' ); ?>
			</div>
			<div id="display-settings">
				<?php do_settings_sections( 'mindd_widget_display_settings_section' ); ?>
			</div>
			<div id="branding-settings">
				<?php do_settings_sections( 'mindd_widget_branding_settings_section' ); ?>
			</div>
			<div id="text-settings">
				<h2 class="title"><?php esc_html_e( 'Texts', 'mindd-widget' ); // phpcs:ignore WordPress.WP.I18n.NonSingularStringLiteralText ?></h2>
				<?php foreach ( $this->languages as $key => $value ) : ?>
					<?php do_settings_sections( 'mindd_widget_text_' . strtolower( $key ) . '_settings_section' ); ?>
				<?php endforeach; ?>
			</div>
		</div>
		<?php
	}

	/**
	 * Save post meta values for the MINDD widget.
	 *
	 * This function checks for specific POST data related to the MINDD widget and updates the post meta accordingly.
	 * Specifically, it updates the 'mindd_widget' and 'mindd_widget_id' meta keys with their respective values.
	 *
	 * @param int $post_id The ID of the post being saved.
	 * @return void
	 */
	public function save_post_meta( $post_id ) {
		// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		if ( ! isset( $_POST['mindd_widget_nonce'] ) || ! wp_verify_nonce( wp_unslash( $_POST['mindd_widget_nonce'] ), 'mindd_widget_meta_box' ) ) {
			return;
		}

		$mindd_widget = isset( $_POST['mindd_widget'] ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST['mindd_widget'] ) ) : array();

		if ( isset( $_POST['mindd_widget'] ) ) {
			update_post_meta( $post_id, 'mindd_widget', $mindd_widget );
		}

		$mindd_widget_id = isset( $_POST['mindd_widget_id'] ) ? sanitize_text_field( wp_unslash( $_POST['mindd_widget_id'] ) ) : '';

		if ( isset( $_POST['mindd_widget_id'] ) ) {
			update_post_meta( $post_id, 'mindd_widget_id', $mindd_widget_id );
		}
	}
}
