<?php

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current version of the plugin.
 *
 * @package    Mindd_Widget
 * @subpackage Mindd_Widget/includes
 */
class Mindd_Widget {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power the plugin.
	 *
	 * @var Mindd_Widget_Loader
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @var string
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @var string
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and the public-facing side of the site.
	 */
	public function __construct() {
		if ( defined( 'MINDD_WIDGET_PLUGIN_VERSION' ) ) {
			$this->version = MINDD_WIDGET_PLUGIN_VERSION;
		} else {
			$this->version = '2.0.0';
		}
		$this->plugin_name = 'mindd-widget';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_public_hooks();
		$this->define_admin_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Mindd_Widget_Loader. Orchestrates the hooks of the plugin.
	 * - Mindd_Widget_i18n. Defines internationalization functionality.
	 * - Mindd_Widget_Admin. Defines all hooks for the admin area.
	 *
	 * Create an instance of the loader which will be used to register the hooks with WordPress.
	 *
	 * @return void
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-mindd-widget-loader.php';

		/**
		 * The class responsible for defining internationalization functionality of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-mindd-widget-i18n.php';

		/**
		 * The class responsible for updates via the wp stockroom.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-stockroom-updater.php';

		/**
		 * The class responsible for the settings page.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-mindd-widget-settings.php';

		/**
		 * The class responsible for defining all actions that occur in the admin-facing side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-mindd-widget-admin.php';

		/**
		 * The class responsible for the shortcode.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'dynamic-block.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-mindd-widget-public.php';

		/**
		 * The class responsible for the shortcode.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-mindd-widget-shortcode.php';

		$this->loader = new Mindd_Widget_Loader();
	}

	/**
	 * Register all of the hooks related to the public-facing functionality of the plugin.
	 *
	 * @return void
	 */
	private function define_public_hooks() {
		$plugin_public = new Mindd_Widget_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_action( 'init', new Mindd_Widget_Shortcode(), 'add_shortcode' );
	}

	/**
	 * Register all of the hooks related to the admin-facing functionality of the plugin.
	 *
	 * @return void
	 */
	private function define_admin_hooks() {
		$plugin_admin = new Mindd_Widget_Admin( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );

		$this->loader->add_filter( 'update_plugins_stockroom.mindd.dev', new WP_Stockroom_Updater(), 'check_update', 10, 4 );
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Mindd_Widget_i18n class in order to set the domain and to register the hook with WordPress.
	 *
	 * @return void
	 */
	private function set_locale() {
		$plugin_i18n = new Mindd_Widget_I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @return void
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of WordPress and to define internationalization functionality.
	 *
	 * @return string
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @return Mindd_Widget_Loader
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @return string
	 */
	public function get_version() {
		return $this->version;
	}

}
